@@ -43,7 +43,8 @@ class RouteHandler(APIHandler):
             timeout_ms = int(1000 * timeout)
             events = dict(poller.poll(timeout_ms))
             if not events:
-                raise TimeoutError("Timeout waiting for response")
+                self.write(json.dumps({}))
+                break
             if control_socket not in events:
                 continue
             res = await client.control_channel.get_msg(timeout=0)
